#!/usr/bin/python

# This is a simple bot with schedule timer
# https://schedule.readthedocs.io

import threading
import time

import schedule

from telebot import TeleBot

API_TOKEN = "<api_token>"
bot = TeleBot(API_TOKEN)


@bot.message_handler(commands=["help", "start"])
def send_welcome(message):
    bot.reply_to(message, "Hi! Use /set <seconds> to set a timer")


def beep(chat_id) -> None:
    """Send the beep message."""
    bot.send_message(chat_id, text="Beep!")


@bot.message_handler(commands=["set"])
def set_timer(message):
    args = message.text.split()
    if len(args) > 1 and args[1].isdigit():
        sec = int(args[1])
        schedule.every(sec).seconds.do(beep, message.chat.id).tag(message.chat.id)
    else:
        bot.reply_to(message, "Usage: /set <seconds>")


@bot.message_handler(commands=["unset"])
def unset_timer(message):
    schedule.clear(message.chat.id)


if __name__ == "__main__":
    threading.Thread(
        target=bot.infinity_polling, name="bot_infinity_polling", daemon=True
    ).start()
    while True:
        schedule.run_pending()
        time.sleep(1)



import telebot
from telebot import custom_filters

bot = telebot.TeleBot("TOKEN")


# Check if message starts with @admin tag
@bot.message_handler(text_startswith="@admin")
def start_filter(message):
    bot.send_message(message.chat.id, "Looks like you are calling admin, wait...")


# Check if text is hi or hello
@bot.message_handler(text=["hi", "hello"])
def text_filter(message):
    bot.send_message(
        message.chat.id, "Hi, {name}!".format(name=message.from_user.first_name)
    )


# Do not forget to register filters
bot.add_custom_filter(custom_filters.TextMatchFilter())
bot.add_custom_filter(custom_filters.TextStartsFilter())

bot.infinity_polling()



#!/usr/bin/python

# This is a set_my_commands example.
# Press on [/] button in telegram client.
# Important, to update the command menu, be sure to exit the chat with the bot and log in again
# Important, command for chat_id and for group have a higher priority than for all

import asyncio

import telebot
from telebot.async_telebot import AsyncTeleBot

API_TOKEN = "<api_token>"
bot = AsyncTeleBot(API_TOKEN)


async def main():
    # use in for delete with the necessary scope and language_code if necessary
    await bot.delete_my_commands(scope=None, language_code=None)

    await bot.set_my_commands(
        commands=[
            telebot.types.BotCommand("command1", "command1 description"),
            telebot.types.BotCommand("command2", "command2 description"),
        ],
        # scope=telebot.types.BotCommandScopeChat(12345678)  # use for personal command menu for users
        # scope=telebot.types.BotCommandScopeAllPrivateChats()  # use for all private chats
    )

    cmd = await bot.get_my_commands(scope=None, language_code=None)
    print([c.to_json() for c in cmd])


if __name__ == "__main__":
    asyncio.run(main())



#!/usr/bin/python3

# This is a simple bot using message reactions (emoji)
# https://core.telegram.org/bots/api#reactiontype
# https://core.telegram.org/bots/api#update
# allowed_updates: Specify an empty list to receive all update types except, chat_member, message_reaction, and message_reaction_count.
# If you want to receive message_reaction events, you cannot simply leave the allowed_updates=None default value.
# The default list of events does not include chat_member, message_reaction, and message_reaction_count events.
# You must explicitly specify all the events you wish to receive and add message_reaction to this list.
# It’s also important to note that after using allowed_updates=[...], in the future, using allowed_updates=None will mean
# that the list of events you will receive will consist of the events you last explicitly specified.

import random

import telebot
from telebot.types import ReactionTypeEmoji

API_TOKEN = "<api_token>"
bot = telebot.TeleBot(API_TOKEN)


# Send a reactions to all messages with content_type 'text' (content_types defaults to ['text'])
@bot.message_handler(func=lambda message: True)
def send_reaction(message):
    emo = ["\U0001F525", "\U0001F917", "\U0001F60E"]  # or use ["🔥", "🤗", "😎"]
    bot.set_message_reaction(
        message.chat.id,
        message.id,
        [ReactionTypeEmoji(random.choice(emo))],
        is_big=False,
    )


@bot.message_reaction_handler(func=lambda message: True)
def get_reactions(message):
    bot.reply_to(
        message,
        f"You changed the reaction from {[r.emoji for r in message.old_reaction]} to {[r.emoji for r in message.new_reaction]}",
    )


bot.infinity_polling(allowed_updates=["message", "message_reaction"])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This is an example echo bot using webhook with Twisted network framework.
# Updates are received with Twisted web server and processed in reactor thread pool.
# Relevant docs:
# https://twistedmatrix.com/documents/current/core/howto/reactor-basics.html
# https://twistedmatrix.com/documents/current/web/howto/using-twistedweb.html

import json
import logging

from twisted.internet import reactor, ssl
from twisted.web.resource import ErrorPage, Resource
from twisted.web.server import Request, Site

import telebot

API_TOKEN = "<api_token>"

WEBHOOK_HOST = "<ip or hostname>"
WEBHOOK_PORT = 8443  # 443, 80, 88 or 8443 (port need to be 'open')
WEBHOOK_LISTEN = "0.0.0.0"  # In some VPS you may need to put here the IP addr

WEBHOOK_SSL_CERT = "./webhook_cert.pem"  # Path to the ssl certificate
WEBHOOK_SSL_PRIV = "./webhook_pkey.pem"  # Path to the ssl private key

# Quick'n'dirty SSL certificate generation:
#
# openssl genrsa -out webhook_pkey.pem 2048
# openssl req -new -x509 -days 3650 -key webhook_pkey.pem -out webhook_cert.pem
#
# When asked for "Common Name (e.g. server FQDN or YOUR name)" you should reply
# with the same value in you put in WEBHOOK_HOST

WEBHOOK_URL_BASE = "https://{}:{}".format(WEBHOOK_HOST, WEBHOOK_PORT)
WEBHOOK_URL_PATH = "/{}/".format(API_TOKEN)

logger = telebot.logger
telebot.logger.setLevel(logging.INFO)
bot = telebot.TeleBot(API_TOKEN)


# Handle '/start' and '/help'
@bot.message_handler(commands=["help", "start"])
def send_welcome(message):
    bot.reply_to(
        message,
        ("Hi there, I am EchoBot.\n" "I am here to echo your kind words back to you."),
    )


# Handle all other messages
@bot.message_handler(func=lambda message: True, content_types=["text"])
def echo_message(message):
    bot.reply_to(message, message.text)


# Remove webhook, it fails sometimes the set if there is a previous webhook
bot.remove_webhook()

# Set webhook
bot.set_webhook(
    url=WEBHOOK_URL_BASE + WEBHOOK_URL_PATH, certificate=open(WEBHOOK_SSL_CERT, "r")
)


# Process webhook calls
class WebhookHandler(Resource):
    isLeaf = True

    def render_POST(self, request: Request):
        request_body_dict = json.load(request.content)
        update = telebot.types.Update.de_json(request_body_dict)
        reactor.callInThread(lambda: bot.process_new_updates([update]))
        return b""


root = ErrorPage(403, "Forbidden", "")
root.putChild(API_TOKEN.encode(), WebhookHandler())
site = Site(root)
sslcontext = ssl.DefaultOpenSSLContextFactory(WEBHOOK_SSL_PRIV, WEBHOOK_SSL_CERT)
reactor.listenSSL(8443, site, sslcontext)
reactor.run()



# This bot is needed to connect two people and their subsequent anonymous communication
#
# Avaiable commands:
# `/start` - Just send you a messsage how to start
# `/find` - Find a person you can contact
# `/stop` - Stop active conversation

import telebot
from telebot import types

# Initialize bot with your token
bot = telebot.TeleBot("TOKEN")

# The `users` variable is needed to contain chat ids that are either in the search or in the active dialog, like {chat_id, chat_id}
users = {}
# The `freeid` variable is needed to contain chat id, that want to start conversation
# Or, in other words: chat id of user in the search
freeid = None


# `/start` command handler
#
# That command only sends you 'Just use /find command!'
@bot.message_handler(commands=["start"])
def start(message: types.Message):
    bot.send_message(message.chat.id, "Just use /find command!")


# `/find` command handler
#
# That command finds opponent for you
#
# That command according to the following principle:
# 1. You have written `/find` command
# 2. If you are already in the search or have an active dialog, bot sends you 'Shut up!'
# 3. If not:
#   3.1. Bot sends you 'Finding...'
#   3.2. If there is no user in the search:
#       3.2.2. `freeid` updated with `your_chat_id`
#   3.3. If there is user in the search:
#       3.3.1. Both you and the user in the search recieve the message 'Founded!'
#       3.3.2. `users` updated with a {user_in_the_search_chat_id, your_chat_id}
#       3.3.3. `users` updated with a {your_chat_id, user_in_the_search_id}
#       3.3.4. `freeid` updated with `None`
@bot.message_handler(commands=["find"])
def find(message: types.Message):
    global freeid

    if message.chat.id not in users:
        bot.send_message(message.chat.id, "Finding...")

        if freeid is None:
            freeid = message.chat.id
        else:
            # Question:
            # Is there any way to simplify this like `bot.send_message([message.chat.id, freeid], 'Founded!')`?
            bot.send_message(message.chat.id, "Founded!")
            bot.send_message(freeid, "Founded!")

            users[freeid] = message.chat.id
            users[message.chat.id] = freeid
            freeid = None

        print(users, freeid)  # Debug purpose, you can remove that line
    else:
        bot.send_message(message.chat.id, "Shut up!")


# `/stop` command handler
#
# That command stops your current conversation (if it exist)
#
# That command according to the following principle:
# 1. You have written `/stop` command
# 2. If you are not have active dialog or you are not in search, bot sends you 'You are not in search!'
# 3. If you are in active dialog:
#   3.1. Bot sends you 'Stopping...'
#   3.2. Bot sends 'Your opponent is leavin`...' to your opponent
#   3.3. {your_opponent_chat_id, your_chat_id} removes from `users`
#   3.4. {your_chat_id, your_opponent_chat_id} removes from `users`
# 4. If you are only in search:
#   4.1. Bot sends you 'Stopping...'
#   4.2. `freeid` updated with `None`
@bot.message_handler(commands=["stop"])
def stop(message: types.Message):
    global freeid

    if message.chat.id in users:
        bot.send_message(message.chat.id, "Stopping...")
        bot.send_message(users[message.chat.id], "Your opponent is leavin`...")

        del users[users[message.chat.id]]
        del users[message.chat.id]

        print(users, freeid)  # Debug purpose, you can remove that line
    elif message.chat.id == freeid:
        bot.send_message(message.chat.id, "Stopping...")
        freeid = None

        print(users, freeid)  # Debug purpose, you can remove that line
    else:
        bot.send_message(message.chat.id, "You are not in search!")


# message handler for conversation
#
# That handler needed to send message from one opponent to another
# If you are not in `users`, you will recieve a message 'No one can hear you...'
# Otherwise all your messages are sent to your opponent
#
# Questions:
# 1. Is there any way to improve readability like `content_types=['all']`?
# 2. Is there any way to register this message handler only when i found the opponent?
@bot.message_handler(
    content_types=[
        "animation",
        "audio",
        "contact",
        "dice",
        "document",
        "location",
        "photo",
        "poll",
        "sticker",
        "text",
        "venue",
        "video",
        "video_note",
        "voice",
    ]
)
def chatting(message: types.Message):
    if message.chat.id in users:
        bot.copy_message(
            users[message.chat.id], users[users[message.chat.id]], message.id
        )
    else:
        bot.send_message(message.chat.id, "No one can hear you...")


bot.infinity_polling(skip_pending=True)



# -*- coding: utf-8 -*-
"""
This Example will show you an advanced usage of CallbackData.
In this example calendar was implemented
"""
import asyncio
from datetime import date

from filters import bind_filters, calendar_factory, calendar_zoom
from keyboards import EMTPY_FIELD, generate_calendar_days, generate_calendar_months

from telebot import types
from telebot.async_telebot import AsyncTeleBot

API_TOKEN = ""
bot = AsyncTeleBot(API_TOKEN)


@bot.message_handler(commands="start")
async def start_command_handler(message: types.Message):
    await bot.send_message(
        message.chat.id,
        f"Hello {message.from_user.first_name}. This bot is an example of calendar keyboard."
        "\nPress /calendar to see it.",
    )


@bot.message_handler(commands="calendar")
async def calendar_command_handler(message: types.Message):
    now = date.today()
    await bot.send_message(
        message.chat.id,
        "Calendar",
        reply_markup=generate_calendar_days(year=now.year, month=now.month),
    )


@bot.callback_query_handler(func=None, calendar_config=calendar_factory.filter())
async def calendar_action_handler(call: types.CallbackQuery):
    callback_data: dict = calendar_factory.parse(callback_data=call.data)
    year, month = int(callback_data["year"]), int(callback_data["month"])

    await bot.edit_message_reply_markup(
        call.message.chat.id,
        call.message.id,
        reply_markup=generate_calendar_days(year=year, month=month),
    )


@bot.callback_query_handler(func=None, calendar_zoom_config=calendar_zoom.filter())
async def calendar_zoom_out_handler(call: types.CallbackQuery):
    callback_data: dict = calendar_zoom.parse(callback_data=call.data)
    year = int(callback_data.get("year"))

    await bot.edit_message_reply_markup(
        call.message.chat.id,
        call.message.id,
        reply_markup=generate_calendar_months(year=year),
    )


@bot.callback_query_handler(func=lambda call: call.data == EMTPY_FIELD)
async def callback_empty_field_handler(call: types.CallbackQuery):
    await bot.answer_callback_query(call.id)


if __name__ == "__main__":
    bind_filters(bot)
    asyncio.run(bot.infinity_polling())



from telebot.asyncio_storage.base_storage import StateContext, StateStorageBase


class StateMemoryStorage(StateStorageBase):
    def __init__(self) -> None:
        self.data = {}
        #
        # {chat_id: {user_id: {'state': None, 'data': {}}, ...}, ...}

    async def set_state(self, chat_id, user_id, state):
        if hasattr(state, "name"):
            state = state.name
        if chat_id in self.data:
            if user_id in self.data[chat_id]:
                self.data[chat_id][user_id]["state"] = state
                return True
            else:
                self.data[chat_id][user_id] = {"state": state, "data": {}}
                return True
        self.data[chat_id] = {user_id: {"state": state, "data": {}}}
        return True

    async def delete_state(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                del self.data[chat_id][user_id]
                if chat_id == user_id:
                    del self.data[chat_id]

                return True

        return False

    async def get_state(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                return self.data[chat_id][user_id]["state"]

        return None

    async def get_data(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                return self.data[chat_id][user_id]["data"]

        return None

    async def reset_data(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                self.data[chat_id][user_id]["data"] = {}
                return True
        return False

    async def set_data(self, chat_id, user_id, key, value):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                self.data[chat_id][user_id]["data"][key] = value
                return True
        raise RuntimeError(
            "chat_id {} and user_id {} does not exist".format(chat_id, user_id)
        )

    def get_interactive_data(self, chat_id, user_id):
        return StateContext(self, chat_id, user_id)

    async def save(self, chat_id, user_id, data):
        self.data[chat_id][user_id]["data"] = data



# Just a little example of middleware handlers

from telebot import TeleBot
from telebot.handler_backends import BaseMiddleware, CancelUpdate

bot = TeleBot(
    "TOKEN", use_class_middlewares=True
)  # if you don't set it to true, middlewares won't work


class SimpleMiddleware(BaseMiddleware):
    def __init__(self, limit) -> None:
        self.last_time = {}
        self.limit = limit
        self.update_types = ["message"]
        # Always specify update types, otherwise middlewares won't work

    def pre_process(self, message, data):
        if not message.from_user.id in self.last_time:
            # User is not in a dict, so lets add and cancel this function
            self.last_time[message.from_user.id] = message.date
            return
        if message.date - self.last_time[message.from_user.id] < self.limit:
            # User is flooding
            bot.send_message(message.chat.id, "You are making request too often")
            return CancelUpdate()
        self.last_time[message.from_user.id] = message.date

    def post_process(self, message, data, exception):
        pass


bot.setup_middleware(SimpleMiddleware(2))


@bot.message_handler(commands=["start"])
def start(message):  # you don't have to put data in handler.
    bot.send_message(message.chat.id, "Hello!")


bot.infinity_polling()



import sys

sys.path.append("../")

REDIS_TESTS = False

import os
import time

import pytest

import telebot
from telebot import types
from telebot.handler_backends import FileHandlerBackend, MemoryHandlerBackend

if REDIS_TESTS:
    from telebot.handler_backends import RedisHandlerBackend


@pytest.fixture()
def telegram_bot():
    return telebot.TeleBot("", threaded=False)


@pytest.fixture
def private_chat():
    return types.Chat(id=11, type="private")


@pytest.fixture
def user():
    return types.User(id=10, is_bot=False, first_name="Some User")


@pytest.fixture()
def message(user, private_chat):
    params = {"text": "/start"}
    return types.Message(
        message_id=1,
        from_user=user,
        date=None,
        chat=private_chat,
        content_type="text",
        options=params,
        json_string="",
    )


@pytest.fixture()
def reply_to_message(user, private_chat, message):
    params = {"text": "/start"}
    reply_message = types.Message(
        message_id=2,
        from_user=user,
        date=None,
        chat=private_chat,
        content_type="text",
        options=params,
        json_string="",
    )
    reply_message.reply_to_message = message
    return reply_message


@pytest.fixture()
def update_type(message):
    edited_message = None
    channel_post = None
    edited_channel_post = None
    inline_query = None
    chosen_inline_result = None
    callback_query = None
    shipping_query = None
    pre_checkout_query = None
    poll = None
    poll_answer = None
    my_chat_member = None
    chat_member = None
    chat_join_request = None
    message_reaction = None
    message_reaction_count = None
    chat_boost = None
    chat_boost_removed = None
    return types.Update(
        1001234038283,
        message,
        edited_message,
        channel_post,
        edited_channel_post,
        inline_query,
        chosen_inline_result,
        callback_query,
        shipping_query,
        pre_checkout_query,
        poll,
        poll_answer,
        my_chat_member,
        chat_member,
        chat_join_request,
        message_reaction,
        message_reaction_count,
        chat_boost,
        chat_boost_removed,
        None,
        None,
        None,
        None,
    )


@pytest.fixture()
def reply_to_message_update_type(reply_to_message):
    edited_message = None
    channel_post = None
    edited_channel_post = None
    inline_query = None
    chosen_inline_result = None
    callback_query = None
    shipping_query = None
    pre_checkout_query = None
    poll = None
    poll_answer = None
    my_chat_member = None
    chat_member = None
    chat_join_request = None
    message_reaction = None
    message_reaction_count = None
    chat_boost = None
    chat_boost_removed = None
    return types.Update(
        1001234038284,
        reply_to_message,
        edited_message,
        channel_post,
        edited_channel_post,
        inline_query,
        chosen_inline_result,
        callback_query,
        shipping_query,
        pre_checkout_query,
        poll,
        poll_answer,
        my_chat_member,
        chat_member,
        chat_join_request,
        message_reaction,
        message_reaction_count,
        chat_boost,
        chat_boost_removed,
        None,
        None,
        None,
        None,
    )


def next_handler(message):
    message.text = "entered next_handler"


def test_memory_handler_backend_default_backend(telegram_bot):
    assert telegram_bot.reply_backend.__class__ == MemoryHandlerBackend
    assert telegram_bot.next_step_backend.__class__ == MemoryHandlerBackend


def test_memory_handler_backend_register_next_step_handler(
    telegram_bot, private_chat, update_type
):
    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_next_step_handler_by_chat_id(
            message.chat.id, next_handler
        )

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    assert len(telegram_bot.next_step_backend.handlers[private_chat.id]) == 1

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered next_handler"

    assert private_chat.id not in telegram_bot.next_step_backend.handlers


def test_memory_handler_backend_clear_next_step_handler(
    telegram_bot, private_chat, update_type
):
    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_next_step_handler_by_chat_id(
            message.chat.id, next_handler
        )

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    assert len(telegram_bot.next_step_backend.handlers[private_chat.id]) == 1

    telegram_bot.clear_step_handler_by_chat_id(private_chat.id)

    assert private_chat.id not in telegram_bot.next_step_backend.handlers

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"


def test_memory_handler_backend_register_reply_handler(
    telegram_bot, private_chat, update_type, reply_to_message_update_type
):
    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_for_reply_by_message_id(message.message_id, next_handler)

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    assert len(telegram_bot.reply_backend.handlers[update_type.message.message_id]) == 1

    telegram_bot.process_new_updates([reply_to_message_update_type])
    assert reply_to_message_update_type.message.text == "entered next_handler"

    assert private_chat.id not in telegram_bot.reply_backend.handlers


def test_memory_handler_backend_clear_reply_handler(
    telegram_bot, private_chat, update_type, reply_to_message_update_type
):
    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_for_reply_by_message_id(message.message_id, next_handler)

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    assert len(telegram_bot.reply_backend.handlers[update_type.message.message_id]) == 1

    telegram_bot.clear_reply_handlers_by_message_id(update_type.message.message_id)

    assert update_type.message.message_id not in telegram_bot.reply_backend.handlers

    telegram_bot.process_new_updates([reply_to_message_update_type])
    assert reply_to_message_update_type.message.text == "entered start"


def test_file_handler_backend_register_next_step_handler(
    telegram_bot, private_chat, update_type
):
    telegram_bot.next_step_backend = FileHandlerBackend(
        filename="./.handler-saves/step1.save", delay=0.1
    )

    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_next_step_handler_by_chat_id(
            message.chat.id, next_handler
        )

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    time.sleep(0.2)

    assert os.path.exists(telegram_bot.next_step_backend.filename)

    assert len(telegram_bot.next_step_backend.handlers[private_chat.id]) == 1

    telegram_bot.next_step_backend.handlers = {}

    telegram_bot.next_step_backend.load_handlers()

    assert len(telegram_bot.next_step_backend.handlers[private_chat.id]) == 1

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered next_handler"

    assert private_chat.id not in telegram_bot.next_step_backend.handlers

    time.sleep(0.2)
    if os.path.exists(telegram_bot.next_step_backend.filename):
        os.remove(telegram_bot.next_step_backend.filename)


def test_file_handler_backend_clear_next_step_handler(
    telegram_bot, private_chat, update_type
):
    telegram_bot.next_step_backend = FileHandlerBackend(
        filename="./.handler-saves/step2.save", delay=0.1
    )

    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_next_step_handler_by_chat_id(
            message.chat.id, next_handler
        )

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    assert len(telegram_bot.next_step_backend.handlers[private_chat.id]) == 1

    time.sleep(0.2)

    assert os.path.exists(telegram_bot.next_step_backend.filename)

    telegram_bot.clear_step_handler_by_chat_id(private_chat.id)

    time.sleep(0.2)

    telegram_bot.next_step_backend.load_handlers()

    assert private_chat.id not in telegram_bot.next_step_backend.handlers

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    time.sleep(0.2)
    if os.path.exists(telegram_bot.next_step_backend.filename):
        os.remove(telegram_bot.next_step_backend.filename)


def test_redis_handler_backend_register_next_step_handler(
    telegram_bot, private_chat, update_type
):
    if not REDIS_TESTS:
        pytest.skip(
            "please install redis and configure redis server, then enable REDIS_TESTS"
        )

    telegram_bot.next_step_backend = RedisHandlerBackend(
        prefix="pyTelegramBotApi:step_backend1"
    )

    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_next_step_handler_by_chat_id(
            message.chat.id, next_handler
        )

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered next_handler"


def test_redis_handler_backend_clear_next_step_handler(
    telegram_bot, private_chat, update_type
):
    if not REDIS_TESTS:
        pytest.skip(
            "please install redis and configure redis server, then enable REDIS_TESTS"
        )

    telegram_bot.next_step_backend = RedisHandlerBackend(
        prefix="pyTelegramBotApi:step_backend2"
    )

    @telegram_bot.message_handler(commands=["start"])
    def start(message):
        message.text = "entered start"
        telegram_bot.register_next_step_handler_by_chat_id(
            message.chat.id, next_handler
        )

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"

    telegram_bot.clear_step_handler_by_chat_id(private_chat.id)

    telegram_bot.process_new_updates([update_type])
    assert update_type.message.text == "entered start"



# Just a little example of middleware handlers

from telebot.async_telebot import AsyncTeleBot
from telebot.asyncio_handler_backends import BaseMiddleware, CancelUpdate

bot = AsyncTeleBot("TOKEN")


class SimpleMiddleware(BaseMiddleware):
    def __init__(self, limit) -> None:
        self.last_time = {}
        self.limit = limit
        self.update_types = ["message"]
        # Always specify update types, otherwise middlewares won't work

    async def pre_process(self, message, data):
        if not message.from_user.id in self.last_time:
            # User is not in a dict, so lets add and cancel this function
            self.last_time[message.from_user.id] = message.date
            return
        if message.date - self.last_time[message.from_user.id] < self.limit:
            # User is flooding
            await bot.send_message(message.chat.id, "You are making request too often")
            return CancelUpdate()
        self.last_time[message.from_user.id] = message.date

    async def post_process(self, message, data, exception):
        pass


bot.setup_middleware(SimpleMiddleware(2))


@bot.message_handler(commands=["start"])
async def start(message):
    await bot.send_message(message.chat.id, "Hello!")


import asyncio

asyncio.run(bot.polling())



import telebot
from telebot.async_telebot import AsyncTeleBot

bot = AsyncTeleBot("TOKEN")


@bot.message_handler(content_types=["photo"])
async def new_message(message: telebot.types.Message):
    result_message = await bot.send_message(
        message.chat.id,
        "<i>Downloading your photo...</i>",
        parse_mode="HTML",
        disable_web_page_preview=True,
    )
    file_path = await bot.get_file(message.photo[-1].file_id)
    downloaded_file = await bot.download_file(file_path.file_path)
    with open("file.jpg", "wb") as new_file:
        new_file.write(downloaded_file)
    await bot.edit_message_text(
        chat_id=message.chat.id,
        message_id=result_message.id,
        text="<i>Done!</i>",
        parse_mode="HTML",
    )


import asyncio

asyncio.run(bot.polling())



from telebot.types import InlineKeyboardButton, InlineKeyboardMarkup


def languages_keyboard():
    return InlineKeyboardMarkup(
        keyboard=[
            [
                InlineKeyboardButton(text="English", callback_data="en"),
                InlineKeyboardButton(text="Русский", callback_data="ru"),
                InlineKeyboardButton(text="O'zbekcha", callback_data="uz_Latn"),
            ]
        ]
    )


def clicker_keyboard(_):
    return InlineKeyboardMarkup(
        keyboard=[
            [
                InlineKeyboardButton(text=_("click"), callback_data="click"),
            ]
        ]
    )



#!/usr/bin/python

# This example shows how to implement deep linking (https://core.telegram.org/bots#deep-linking)
# with the pyTelegramBotAPI.
# Note: This is not a working, production-ready sample.
#
# In this example we are connecting a user account on a website with a Telegram bot.
# Implementing this will enable you to push notifications (and other content) to your users' Telegram account.
# In this explanation the word 'database' can refer to any form of key-value storage.
# The deep linking explained:
#
# 1.	Let the user log in on an actual website with actual username-password authentication.
#
# 2. 	Generate a unique hashcode (we will call it unique_code)
#
# 3. 	Save unique_code->username to the database.
#
# 4. 	Show the user the URL https://telegram.me/YOURBOTNAME?start=unique_code
#
# 5. 	Now as soon as the user opens this URL in Telegram and presses 'Start',
#       your bot will receive a text message containing '/start unique_code',
#       where unique_code is of course replaced by the actual hashcode.
#
# 6. 	Let the bot retrieve the username by querying the database for unique_code.
#
# 7. 	Save chat_id->username to the database.
#
# 8. 	Now when your bot receives another message, it can query message.chat.id in the database
#       to check if the message is from this specific user. (And handle accordingly) or
#       you can push messages to the user using his chat id.
#
# Steps 1 to 4 will have to be implemented in a web server, using a language such as PHP, Python, C# or Java. These
# steps are not shown here. Only steps 5 to 7 are illustrated, some in pseudo-code, with this example.

import telebot

bot = telebot.TeleBot("TOKEN")


def extract_unique_code(text):
    # Extracts the unique_code from the sent /start command.
    return text.split()[1] if len(text.split()) > 1 else None


def in_storage(unique_code):
    # (pseudo-code) Should check if a unique code exists in storage
    return True


def get_username_from_storage(unique_code):
    # (pseudo-code) Does a query to the storage, retrieving the associated username
    # Should be replaced by a real database-lookup.
    return "ABC" if in_storage(unique_code) else None


def save_chat_id(chat_id, username):
    # (pseudo-code) Save the chat_id->username to storage
    # Should be replaced by a real database query.
    pass


@bot.message_handler(commands=["start"])
def send_welcome(message):
    unique_code = extract_unique_code(message.text)
    if unique_code:  # if the '/start' command contains a unique_code
        username = get_username_from_storage(unique_code)
        if username:  # if the username exists in our database
            save_chat_id(message.chat.id, username)
            reply = "Hello {0}, how are you?".format(username)
        else:
            reply = "I have no clue who you are..."
    else:
        reply = "Please visit me via a provided URL from the website."
    bot.reply_to(message, reply)


bot.infinity_polling()



# This example show how to use inline keyboards and process button presses
import telebot
from telebot.types import InlineKeyboardButton, InlineKeyboardMarkup

TELEGRAM_TOKEN = "<TOKEN>"

bot = telebot.TeleBot(TELEGRAM_TOKEN)


def gen_markup():
    markup = InlineKeyboardMarkup()
    markup.row_width = 2
    markup.add(
        InlineKeyboardButton("Yes", callback_data="cb_yes"),
        InlineKeyboardButton("No", callback_data="cb_no"),
    )
    return markup


@bot.callback_query_handler(func=lambda call: True)
def callback_query(call):
    if call.data == "cb_yes":
        bot.answer_callback_query(call.id, "Answer is Yes")
    elif call.data == "cb_no":
        bot.answer_callback_query(call.id, "Answer is No")


@bot.message_handler(func=lambda message: True)
def message_handler(message):
    bot.send_message(message.chat.id, "Yes/no?", reply_markup=gen_markup())


bot.infinity_polling()



import json

from telebot.storage.base_storage import StateContext, StateStorageBase

redis_installed = True
try:
    from redis import ConnectionPool, Redis

except:
    redis_installed = False


class StateRedisStorage(StateStorageBase):
    """
    This class is for Redis storage.
    This will work only for states.
    To use it, just pass this class to:
    TeleBot(storage=StateRedisStorage())
    """

    def __init__(
        self,
        host="localhost",
        port=6379,
        db=0,
        password=None,
        prefix="telebot_",
        redis_url=None,
    ):
        super().__init__()
        if redis_url:
            self.redis = ConnectionPool.from_url(redis_url)
        else:
            self.redis = ConnectionPool(host=host, port=port, db=db, password=password)
        # self.con = Redis(connection_pool=self.redis) -> use this when necessary
        #
        # {chat_id: {user_id: {'state': None, 'data': {}}, ...}, ...}
        self.prefix = prefix
        if not redis_installed:
            raise Exception(
                "Redis is not installed. Install it via 'pip install redis'"
            )

    def get_record(self, key):
        """
        Function to get record from database.
        It has nothing to do with states.
        Made for backward compatibility
        """
        connection = Redis(connection_pool=self.redis)
        result = connection.get(self.prefix + str(key))
        connection.close()
        if result:
            return json.loads(result)
        return

    def set_record(self, key, value):
        """
        Function to set record to database.
        It has nothing to do with states.
        Made for backward compatibility
        """
        connection = Redis(connection_pool=self.redis)
        connection.set(self.prefix + str(key), json.dumps(value))
        connection.close()
        return True

    def delete_record(self, key):
        """
        Function to delete record from database.
        It has nothing to do with states.
        Made for backward compatibility
        """
        connection = Redis(connection_pool=self.redis)
        connection.delete(self.prefix + str(key))
        connection.close()
        return True

    def set_state(self, chat_id, user_id, state):
        """
        Set state for a particular user in a chat.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if hasattr(state, "name"):
            state = state.name

        if response:
            if user_id in response:
                response[user_id]["state"] = state
            else:
                response[user_id] = {"state": state, "data": {}}
        else:
            response = {user_id: {"state": state, "data": {}}}
        self.set_record(chat_id, response)

        return True

    def delete_state(self, chat_id, user_id):
        """
        Delete state for a particular user in a chat.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                del response[user_id]
                if user_id == str(chat_id):
                    self.delete_record(chat_id)
                    return True
                else:
                    self.set_record(chat_id, response)
                return True
        return False

    def get_value(self, chat_id, user_id, key):
        """
        Get value for a data of a user in a chat.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                if key in response[user_id]["data"]:
                    return response[user_id]["data"][key]
        return None

    def get_state(self, chat_id, user_id):
        """
        Get state of a user in a chat.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                return response[user_id]["state"]

        return None

    def get_data(self, chat_id, user_id):
        """
        Get data of particular user in a particular chat.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                return response[user_id]["data"]
        return None

    def reset_data(self, chat_id, user_id):
        """
        Reset data of a user in a chat.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                response[user_id]["data"] = {}
                self.set_record(chat_id, response)
                return True

    def set_data(self, chat_id, user_id, key, value):
        """
        Set data without interactive data.
        """
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                response[user_id]["data"][key] = value
                self.set_record(chat_id, response)
                return True
        return False

    def get_interactive_data(self, chat_id, user_id):
        """
        Get Data in interactive way.
        You can use with() with this function.
        """
        return StateContext(self, chat_id, user_id)

    def save(self, chat_id, user_id, data):
        response = self.get_record(chat_id)
        user_id = str(user_id)
        if response:
            if user_id in response:
                response[user_id]["data"] = data
                self.set_record(chat_id, response)
                return True



from telebot.async_telebot import AsyncTeleBot
from telebot.asyncio_handler_backends import ContinueHandling

bot = AsyncTeleBot("TOKEN")


@bot.message_handler(commands=["start"])
async def start(message):
    await bot.send_message(message.chat.id, "Hello World!")
    return ContinueHandling()


@bot.message_handler(commands=["start"])
async def start2(message):
    """
    This handler comes after the first one, but it will never be called.
    But you can call it by returning ContinueHandling() in the first handler.

    If you return ContinueHandling() in the first handler, the next
    registered handler with appropriate filters will be called.
    """
    await bot.send_message(message.chat.id, "Hello World2!")


import asyncio

asyncio.run(bot.polling())  # just a reminder that infinity polling
# wraps polling into try/except block just as sync version,
# but you can use any of them because neither of them stops if you
# pass non_stop=True



#!/usr/bin/python

# This is a simple echo bot using the decorator mechanism.
# It echoes any incoming text messages.

import telebot

API_TOKEN = "<api_token>"

bot = telebot.TeleBot(API_TOKEN)


# Handle '/start' and '/help'
@bot.message_handler(commands=["help", "start"])
def send_welcome(message):
    bot.reply_to(
        message,
        """\
Hi there, I am EchoBot.
I am here to echo your kind words back to you. Just say anything nice and I'll say the exact same thing to you!\
""",
    )


# Handle all other messages with content_type 'text' (content_types defaults to ['text'])
@bot.message_handler(func=lambda message: True)
def echo_message(message):
    bot.reply_to(message, message.text)


bot.infinity_polling()



#!/usr/bin/python

# This example shows how to implement i18n (internationalization) l10n (localization) to create
# multi-language bots with middleware handler.
#
# Note: For the sake of simplicity of this example no extra library is used. However, it is recommended to use
# better i18n systems (gettext and etc) for handling multilingual translations.
# This is not a working, production-ready sample and it is highly recommended not to use it in production.
#
# In this example let's imagine we want to introduce localization or internationalization into our project and
# we need some global function to activate the language once and to use that language in all other message
# handler functions for not repeatedly activating it.
# The middleware (i18n and l10n) is explained:

import telebot
from telebot import apihelper

apihelper.ENABLE_MIDDLEWARE = True

TRANSLATIONS = {"hello": {"en": "hello", "ru": "привет", "uz": "salom"}}

_lang = "en"


def activate(lang):
    global _lang
    _lang = lang


def _(string):
    return TRANSLATIONS[string][_lang]


bot = telebot.TeleBot("TOKEN")


@bot.middleware_handler(update_types=["message"])
def activate_language(bot_instance, message):
    activate(message.from_user.language_code)


@bot.message_handler(commands=["start"])
def start(message):
    bot.send_message(message.chat.id, _("hello"))


bot.infinity_polling()



# -*- coding: utf-8 -*-
import sys

sys.path.append("../")
from telebot import types


def test_json_user():
    jsonstring = r'{"id":101176298,"first_name":"RDSSBOT","last_name":")))","username":"rdss_bot","is_bot":true, "is_premium":true, "added_to_attachment_menu": true}'
    u = types.User.de_json(jsonstring)
    assert u.id == 101176298
    assert u.full_name == "RDSSBOT )))"
    assert u.is_premium is True
    assert u.added_to_attachment_menu is True


def test_json_message():
    jsonstring = r'{"message_id":1,"from":{"id":108929734,"first_name":"Frank","last_name":"Wang","username":"eternnoir","is_bot":true},"chat":{"id":1734,"first_name":"F","type":"private","last_name":"Wa","username":"oir"},"date":1435296025,"text":"HIHI"}'
    msg = types.Message.de_json(jsonstring)
    assert msg.text == "HIHI"


def test_json_message_with_reply_markup():
    jsonstring = r'{"message_id":48,"from":{"id":153587469,"is_bot":false,"first_name":"Neko","username":"Neko"},"chat":{"id":153587469,"first_name":"Neko","username":"Neko","type":"private"},"date":1598879570,"text":"test","reply_markup":{"inline_keyboard":[[{"text":"Google","url":"http://www.google.com"},{"text":"Yahoo","url":"http://www.yahoo.com"}]]}}'
    msg = types.Message.de_json(jsonstring)
    assert msg.content_type == "text"
    assert msg.reply_markup.keyboard[0][0].text == "Google"


def test_json_InlineKeyboardMarkup():
    jsonstring = r'{"inline_keyboard":[[{"text":"Google","url":"http://www.google.com"},{"text":"Yahoo","url":"http://www.yahoo.com"}]]}'
    markup = types.InlineKeyboardMarkup.de_json(jsonstring)
    assert markup.keyboard[0][0].text == "Google"
    assert markup.keyboard[0][1].url == "http://www.yahoo.com"


def test_json_InlineKeyboardButton():
    jsonstring = r'{"text":"Google","url":"http://www.google.com"}'
    button = types.InlineKeyboardButton.de_json(jsonstring)
    assert button.text == "Google"
    assert button.url == "http://www.google.com"


def test_json_message_with_dice():
    jsonstring = r'{"message_id":5560,"from":{"id":879343317,"is_bot":false,"first_name":"George","last_name":"Forse","username":"dr_fxrse","language_code":"ru"},"chat":{"id":879343317,"first_name":"George","last_name":"Forse","username":"dr_fxrse","type":"private"},"date":1586926330,"dice":{"value": 4, "emoji": "\ud83c\udfaf"}}'
    msg = types.Message.de_json(jsonstring)
    assert msg.content_type == "dice"
    assert isinstance(msg.dice, types.Dice)
    assert msg.dice.value == 4
    assert msg.dice.emoji == "🎯"


def test_json_message_group():
    json_string = r'{"message_id":10,"from":{"id":12345,"first_name":"g","last_name":"G","username":"GG","is_bot":true},"chat":{"id":-866,"type":"private","title":"\u4ea4"},"date":1435303157,"text":"HIHI","has_protected_content":true}'
    msg = types.Message.de_json(json_string)
    assert msg.text == "HIHI"
    assert len(msg.chat.title) != 0
    assert msg.from_user.username == "GG"


def test_json_GroupChat():
    json_string = r'{"id":8926,"title":"\u5c4e\u4f2f\u98ef\u98ef\u4e4b\u4ea4"}'
    gc = types.GroupChat.de_json(json_string)
    assert gc.id == 8926


def test_json_Document():
    json_string = r'{"file_name":"Text File","thumbnail":{},"file_id":"BQADBQADMwIAAsYifgZ_CEh0u682xwI","file_unique_id": "AgADJQEAAqfhOEY","file_size":446}'
    doc = types.Document.de_json(json_string)
    assert doc.thumbnail is None
    assert doc.file_name == "Text File"


def test_json_Message_Audio():
    json_string = r'{"message_id":131,"from":{"id":12775,"first_name":"dd","username":"dd","is_bot":true },"chat":{"id":10834,"first_name":"dd","type":"private","type":"private","last_name":"dd","username":"dd"},"date":1439978364,"audio":{"duration":1,"mime_type":"audio\/mpeg","title":"pyTelegram","performer":"eternnoir","file_id":"BQADBQADDH1JaB8-1KyWUss2-Ag","file_unique_id": "AgADawEAAn8VSFY","file_size":20096}}'
    msg = types.Message.de_json(json_string)
    assert msg.audio.duration == 1
    assert msg.content_type == "audio"
    assert msg.audio.performer == "eternnoir"
    assert msg.audio.title == "pyTelegram"


def test_json_Message_Sticker():
    json_string = r'{"message_id": 21552, "from": {"id": 590740002, "is_bot": false, "first_name": "⚜️ Ƥυrуα ⚜️", "username": "Purya", "language_code": "en"}, "chat": {"id": -1001309982000, "title": "123", "type": "supergroup"}, "date": 1594068909, "sticker": {"type": "regular", "width": 368, "height": 368, "emoji": "🤖", "set_name": "ipuryapack", "is_animated": false, "is_video": true, "thumbnail": {"file_id": "AAMCBAADHQJOFL7mAAJUMF8Dj62hpmDhpRAYvkc8CtIqipolAAJ8AAPA-8cF9yxjgjkLS97A0D4iXQARtQAHbQADHy4AAhoE", "file_unique_id": "AQADwNA-Il0AAx8uAAI", "file_size": 7776, "width": 60, "height": 60}, "file_id": "CAACAgQAAx0CThS-5gACVDBfA4-toaZg4aUQGL5HWerSKoqaJQACArADwPvHBfcsY4I5C3feGgQ", "file_unique_id": "AgADfAADsPvHWQ", "file_size": 14602}}'
    msg = types.Message.de_json(json_string)
    assert msg.sticker.height == 368
    assert msg.sticker.thumbnail.height == 60
    assert msg.content_type == "sticker"


def test_json_Message_Sticker_without_thumbnail():
    json_string = r'{"message_id": 21552, "from": {"id": 590740002, "is_bot": false, "first_name": "⚜️ Ƥυrуα ⚜️", "username": "Purya", "language_code": "en"}, "chat": {"id": -1001309982000, "title": "123", "type": "supergroup"}, "date": 1594068909, "sticker": {"type": "regular", "width": 368, "height": 368, "emoji": "🤖", "set_name": "ipuryapack", "is_animated": false, "is_video": true, "file_id": "CAACAgQAAx0CThS-5gACVDBfA4-toaZg4aUQGL5HWerSKoqaJQACArADwPvHBfcsY4I5C3feGgQ", "file_unique_id": "AgADfAADsPvHWQ", "file_size": 14602}}'
    msg = types.Message.de_json(json_string)
    assert msg.sticker.height == 368
    assert msg.sticker.thumbnail is None
    assert msg.content_type == "sticker"


def test_json_Message_Document():
    json_string = r'{"message_id":97,"from":{"id":10734,"first_name":"Fd","last_name":"Wd","username":"dd","is_bot":true },"chat":{"id":10,"first_name":"Fd","type":"private","last_name":"Wd","username":"dd"},"date":1435478744,"document":{"file_name":"Text File","thumbnail":{},"file_id":"BQADBQADMwIAAsYifgZ_CEh0u682xwI","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":446}}'
    msg = types.Message.de_json(json_string)
    assert msg.document.file_name == "Text File"
    assert msg.content_type == "document"


def test_json_Message_Photo():
    json_string = r'{"message_id":96,"from":{"id":109734,"first_name":"Fd","last_name":"Wd","username":"dd","is_bot":true },"chat":{"id":10734,"first_name":"Fd","type":"private","last_name":"dd","username":"dd"},"date":1435478191,"photo":[{"file_id":"AgADBQADIagxG8YifgYv8yLSj76i-dd","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":615,"width":90,"height":67},{"file_id":"AgADBQADIagxG8YifgYv8yLSj76i-dd","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":10174,"width":320,"height":240},{"file_id":"dd-A_LsTIABFNx-FUOaEa_3AABAQABAg","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":53013,"width":759,"height":570}]}'
    msg = types.Message.de_json(json_string)
    assert len(msg.photo) == 3
    assert msg.content_type == "photo"


def test_json_Message_Video():
    json_string = r'{"message_id":101,"from":{"id":109734,"first_name":"dd","last_name":"dd","username":"dd","is_bot":true },"chat":{"id":109734,"first_name":"dd","type":"private","last_name":"dd","username":"dd"},"date":1435481960,"video":{"duration":3,"caption":"","width":360,"height":640,"thumbnail":{"file_id":"AAQFABPiYnBjkDwMAAIC","file_unique_id": "AQADTeisa3QAAz1nAAI","file_size":1597,"width":50,"height":90},"file_id":"BAADBQADNifgb_TOPEKErGoQI","file_unique_id": "AgADbgEAAn8VSFY","file_size":260699}}'
    msg = types.Message.de_json(json_string)
    assert msg.video
    assert msg.video.duration == 3
    assert msg.video.thumbnail.width == 50
    assert msg.content_type == "video"


def test_json_Message_Location():
    json_string = r'{"message_id":102,"from":{"id":108734,"first_name":"dd","last_name":"dd","username":"dd","is_bot":true },"chat":{"id":1089734,"first_name":"dd","type":"private","last_name":"dd","username":"dd"},"date":1535482469,"location":{"longitude":127.479471,"latitude":26.090577}}'
    msg = types.Message.de_json(json_string)
    assert msg.location.latitude == 26.090577
    assert msg.content_type == "location"


def test_json_UserProfilePhotos():
    json_string = r'{"total_count":1,"photos":[[{"file_id":"AgADAgADqacxG6wpRwABvEB6fpeIcKS4HAIkAATZH_SpyZjzIwdVAAIC","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":6150,"width":160,"height":160},{"file_id":"AgADAgADqacxG6wpRwABvEB6fpeIcKS4HAIkAATOiTNi_YoJMghVAAIC","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":13363,"width":320,"height":320},{"file_id":"AgADAgADqacxG6wpRwABvEB6fpeIcKS4HAIkAAQW4DyFv0-lhglVAAIC","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":28347,"width":640,"height":640},{"file_id":"AgADAgADqacxG6wpRwABvEB6fpeIcKS4HAIkAAT50RvJCg0GQApVAAIC","file_unique_id": "AQAD_QIfa3QAAyA4BgAB","file_size":33953,"width":800,"height":800}]]}'
    upp = types.UserProfilePhotos.de_json(json_string)
    assert upp.photos[0][0].width == 160
    assert upp.photos[0][-1].height == 800


def test_json_contact():
    json_string = r'{"phone_number":"00011111111","first_name":"dd","last_name":"ddl","user_id":8633,"vcard":"SomeContactString"}'
    contact = types.Contact.de_json(json_string)
    assert contact.first_name == "dd"
    assert contact.last_name == "ddl"


def test_json_voice():
    json_string = r'{"duration": 0,"mime_type": "audio/ogg","file_id": "AwcccccccDH1JaB7w_gyFjYQxVAg","file_unique_id": "AgADbAEAAn8VSFY","file_size": 10481}'
    voice = types.Voice.de_json(json_string)
    assert voice.duration == 0
    assert voice.file_size == 10481


def test_json_update():
    json_string = r'{"update_id":938203,"message":{"message_id":241,"from":{"is_bot":true,"id":9734,"first_name":"Fk","last_name":"Wg","username":"nir"},"chat":{"id":1111,"first_name":"Fk","type":"private","last_name":"Wg","username":"oir"},"date":1441447009,"text":"HIHI"}}'
    update = types.Update.de_json(json_string)
    assert update.update_id == 938203
    assert update.message.message_id == 241
    assert update.message.from_user.id == 9734


def test_json_chat():
    json_string = r'{"id": -111111,"title": "Test Title","type": "group", "join_to_send_messages": true, "join_by_request": true}'
    chat = types.Chat.de_json(json_string)
    assert chat.id == -111111
    assert chat.type == "group"
    assert chat.title == "Test Title"
    assert chat.join_to_send_messages is True
    assert chat.join_by_request is True


def test_InlineQueryResultCachedPhoto():
    iq = types.InlineQueryResultCachedPhoto("aaa", "Fileid")
    json_str = iq.to_json()
    assert "aa" in json_str
    assert "Fileid" in json_str
    assert "caption" not in json_str


def test_InlineQueryResultCachedPhoto_with_title():
    iq = types.InlineQueryResultCachedPhoto("aaa", "Fileid", title="Title")
    json_str = iq.to_json()
    assert "aa" in json_str
    assert "Fileid" in json_str
    assert "Title" in json_str
    assert "caption" not in json_str


def test_InlineQueryResultCachedPhoto_with_markup():
    markup = types.InlineKeyboardMarkup()
    markup.add(types.InlineKeyboardButton("Google", url="http://www.google.com"))
    markup.add(types.InlineKeyboardButton("Yahoo", url="http://www.yahoo.com"))
    iq = types.InlineQueryResultCachedPhoto(
        "aaa", "Fileid", title="Title", reply_markup=markup
    )
    json_str = iq.to_json()
    assert "aa" in json_str
    assert "Fileid" in json_str
    assert "Title" in json_str
    assert "caption" not in json_str
    assert "reply_markup" in json_str


def test_json_poll_1():
    jsonstring = r'{"message_id": 395020,"from": {"id": 111,"is_bot": false,"first_name": "FN","last_name": "LN","username": "Badiboy","language_code": "ru"},"chat": {"id": 111,"first_name": "FN","last_name": "LN","username": "Badiboy","type": "private"},"date": 1587841239,"poll": {"id": "5272018969396510722","question": "Test poll 1","options": [{"text": "Answer 1","voter_count": 0},{"text": "Answer 2","voter_count": 0}],"total_voter_count": 0,"is_closed": false,"is_anonymous": true,"type": "regular","allows_multiple_answers": true}}'
    msg = types.Message.de_json(jsonstring)
    assert msg.poll is not None
    assert isinstance(msg.poll, types.Poll)
    assert msg.poll.id == "5272018969396510722"
    assert msg.poll.question is not None
    assert msg.poll.options is not None
    assert len(msg.poll.options) == 2
    assert msg.poll.allows_multiple_answers is True


def test_json_poll_answer():
    jsonstring = r'{"poll_id": "5895675970559410186", "user": {"id": 329343347, "is_bot": false, "first_name": "Test", "username": "test_user", "last_name": "User", "language_code": "en"}, "option_ids": [1]}'
    __import__("pprint").pprint(__import__("json").loads(jsonstring))
    poll_answer = types.PollAnswer.de_json(jsonstring)
    assert poll_answer.poll_id == "5895675970559410186"
    assert isinstance(poll_answer.user, types.User)
    assert poll_answer.option_ids == [1]


def test_KeyboardButtonPollType():
    markup = types.ReplyKeyboardMarkup()
    markup.add(
        types.KeyboardButton(
            "send me a poll", request_poll=types.KeyboardButtonPollType(type="quiz")
        )
    )
    json_str = markup.to_json()
    assert "request_poll" in json_str
    assert "quiz" in json_str


def test_json_chat_invite_link():
    json_string = r'{"invite_link":"https://t.me/joinchat/MeASP-Wi...","creator":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"ru"},"pending_join_request_count":1,"creates_join_request":true,"is_primary":false,"is_revoked":false}'
    invite_link = types.ChatInviteLink.de_json(json_string)
    assert invite_link.invite_link == "https://t.me/joinchat/MeASP-Wi..."
    assert isinstance(invite_link.creator, types.User)
    assert not invite_link.is_primary
    assert not invite_link.is_revoked
    assert invite_link.expire_date is None
    assert invite_link.member_limit is None
    assert invite_link.name is None
    assert invite_link.creator.id == 927266710
    assert invite_link.pending_join_request_count == 1
    assert invite_link.creates_join_request


def test_chat_member_updated():
    json_string = r'{"chat": {"id": -1234567890123, "type": "supergroup", "title": "No Real Group", "username": "NoRealGroup"}, "from": {"id": 133869498, "is_bot": false, "first_name": "Vincent"}, "date": 1624119999, "old_chat_member": {"user": {"id": 77777777, "is_bot": false, "first_name": "Pepe"}, "status": "member"}, "new_chat_member": {"user": {"id": 77777777, "is_bot": false, "first_name": "Pepe"}, "status": "administrator"}}'
    cm_updated = types.ChatMemberUpdated.de_json(json_string)
    assert cm_updated.chat.id == -1234567890123
    assert cm_updated.from_user.id == 133869498
    assert cm_updated.date == 1624119999
    assert cm_updated.old_chat_member.status == "member"
    assert cm_updated.new_chat_member.status == "administrator"


def test_webhook_info():
    json_string = r'{"url": "https://example.com/webhook", "has_custom_certificate": true, "pending_update_count": 1, "last_error_date": 0, "last_error_message": "", "last_synchronization_error_date": 489309, "max_connections": 40, "allowed_updates": ["message"]}'
    webhook_info = types.WebhookInfo.de_json(json_string)
    print(webhook_info)
    assert webhook_info.url == "https://example.com/webhook"
    assert webhook_info.has_custom_certificate is True
    assert webhook_info.pending_update_count == 1
    assert webhook_info.last_error_date == 0
    assert webhook_info.last_error_message == ""
    assert webhook_info.max_connections == 40
    assert webhook_info.last_synchronization_error_date == 489309
    assert webhook_info.allowed_updates == ["message"]


def test_sent_web_app_message():
    json_string = r'{"inline_message_id": "29430"}'
    sent_web_app_message = types.SentWebAppMessage.de_json(json_string)
    assert sent_web_app_message.inline_message_id == "29430"


def test_message_entity():
    # TODO: Add support for nesting entities

    sample_string_1 = r'{"update_id":934522126,"message":{"message_id":1374510,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682177590,"text":"b b b","entities":[{"offset":0,"length":2,"type":"bold"},{"offset":0,"length":1,"type":"italic"},{"offset":2,"length":2,"type":"bold"},{"offset":2,"length":1,"type":"italic"},{"offset":4,"length":1,"type":"bold"},{"offset":4,"length":1,"type":"italic"}]}}'
    update = types.Update.de_json(sample_string_1)
    message: types.Message = update.message
    assert message.html_text == "<i><b>b </b></i><i><b>b </b></i><i><b>b</b></i>"

    sample_string_2 = r'{"update_id":934522166,"message":{"message_id":1374526,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682179716,"text":"b b b","entities":[{"offset":0,"length":1,"type":"bold"},{"offset":2,"length":1,"type":"bold"},{"offset":4,"length":1,"type":"italic"}]}}'
    message_2 = types.Update.de_json(sample_string_2).message
    assert message_2.html_text == "<b>b</b> <b>b</b> <i>b</i>"

    sample_string_3 = r'{"update_id":934522172,"message":{"message_id":1374530,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682179968,"text":"This is a bold text with a nested italic and bold text.","entities":[{"offset":10,"length":4,"type":"bold"},{"offset":27,"length":7,"type":"italic"},{"offset":34,"length":15,"type":"bold"},{"offset":34,"length":15,"type":"italic"}]}}'
    message_3 = types.Update.de_json(sample_string_3).message
    assert (
        message_3.html_text
        == "This is a <b>bold</b> text with a <i>nested </i><i><b>italic and bold</b></i> text."
    )

    sample_string_4 = r'{"update_id":934522437,"message":{"message_id":1374619,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682189507,"forward_from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"forward_date":1682189124,"text":"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa😋😋","entities":[{"offset":0,"length":76,"type":"bold"},{"offset":0,"length":76,"type":"italic"},{"offset":0,"length":76,"type":"underline"},{"offset":0,"length":76,"type":"strikethrough"},{"offset":76,"length":2,"type":"custom_emoji","custom_emoji_id":"5456188142006575553"},{"offset":78,"length":2,"type":"custom_emoji","custom_emoji_id":"5456188142006575553"}]}}'
    message_4 = types.Update.de_json(sample_string_4).message
    assert (
        message_4.html_text
        == '<s><u><i><b>aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa</b></i></u></s><tg-emoji emoji-id="5456188142006575553">😋</tg-emoji><tg-emoji emoji-id="5456188142006575553">😋</tg-emoji>'
    )

    sample_string_5 = r'{"update_id":934522166,"message":{"message_id":1374526,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682179716,"text":"b <b>b</b> <i>i</i>","entities":[{"offset":0,"length":1,"type":"bold"}]}}'
    message_5 = types.Update.de_json(sample_string_5).message
    assert message_5.html_text == "<b>b</b> &lt;b&gt;b&lt;/b&gt; &lt;i&gt;i&lt;/i&gt;"

    sample_string_6 = r'{"update_id":934522166,"message":{"message_id":1374526,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682179716,"text":"<b>b</b> <i>i</i>"}}'
    message_6 = types.Update.de_json(sample_string_6).message
    assert message_6.html_text == "&lt;b&gt;b&lt;/b&gt; &lt;i&gt;i&lt;/i&gt;"

    sample_string_7 = r'{"update_id":934522167,"message":{"message_id":1374526,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en","is_premium":true},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1682179716,"reply_to_message":{"message_id":1374510,"from":{"id":927266710,"is_bot":false,"first_name":">_run","username":"coder2020","language_code":"en"},"chat":{"id":927266710,"first_name":">_run","username":"coder2020","type":"private"},"date":1712765863,"text":"text @UserName b i s u c p #hashtag https://example.com","entities":[{"offset":5,"length":9,"type":"mention"},{"offset":15,"length":1,"type":"bold"},{"offset":17,"length":1,"type":"italic"},{"offset":19,"length":1,"type":"strikethrough"},{"offset":21,"length":1,"type":"underline"},{"offset":23,"length":1,"type":"code"},{"offset":25,"length":1,"type":"spoiler"},{"offset":27,"length":8,"type":"hashtag"},{"offset":36,"length":19,"type":"url"}],"link_preview_options":{"is_disabled":true}},"quote":{"text":"text @UserName b i s u c p #hashtag https://example.com","entities":[{"offset":15,"length":1,"type":"bold"},{"offset":17,"length":1,"type":"italic"},{"offset":19,"length":1,"type":"strikethrough"},{"offset":21,"length":1,"type":"underline"},{"offset":25,"length":1,"type":"spoiler"}],"position":0,"is_manual":true},"text":"quote reply"}}'
    message_7 = types.Update.de_json(sample_string_7).message
    assert (
        message_7.quote.html_text
        == 'text @UserName <b>b</b> <i>i</i> <s>s</s> <u>u</u> c <span class="tg-spoiler">p</span> #hashtag https://example.com'
    )
